@@ -24,46 +24,75 @@
 ##'
 ##' This is a user-interface for regression-based causal mediation analysis as described in Valeri & VanderWeele 2013 and Valeri & VanderWeele 2015.
 ##'
-##' @param data Data frame containing the relevant variables.
-##' @param yvar A character vector of length 1. Outcome variable name. It should be the time variable for survival outcomes.
+##' @param data Data frame containing the following relevant variables.
+##' @param yvar A character vector of length 1. Outcome variable name. It should be the time variable for the survival outcome.
 ##' @param avar A character vector of length 1. Treatment variable name.
 ##' @param mvar A character vector of length 1. Mediator variable name.
-##' @param cvar A character vector of length > 0. Covariate names. Use \code{NULL} if there is no covariate. However, this is a highly suspicious situation. Even if \code{avar} is randomized, \code{mvar} is not. Thus, there should usually be some confounder(s) to account for the common cause structure (confounding) between \code{mvar} and \code{yvar}.
+##' @param cvar A character vector of length > 0. Covariate names. Use \code{NULL} if there is no covariate. However, this is a highly suspicious situation. Even if \code{avar} is randomized, \code{mvar} is not. Thus, there are usually some confounder(s) to account for the common cause structure (confounding) between \code{mvar} and \code{yvar}. 
+##' @param emm_ac_mreg A character vector of length > 0. Effect modifiers names. The covariate vector in treatment-covariate product term in the mediator model.
+##' @param emm_ac_yreg A character vector of length > 0. Effect modifiers names. The covariate vector in treatment-covariate product term in the outcome model. 
+##' @param emm_mc_yreg A character vector of length > 0. Effect modifiers names. The covariate vector in mediator-covariate product term in outcome model. 
 ##' @param eventvar An character vector of length 1. Only required for survival outcome regression models. Note that the coding is 1 for event and 0 for censoring, following the R survival package convention.
-##' @param a0 A numeric vector of length 1. Reference level of treatment variable that is considered "untreated" or "unexposed".
+##' @param a0 A numeric vector of length 1. The reference level of treatment variable that is considered "untreated" or "unexposed".
 ##' @param a1 A numeric vector of length 1.
 ##' @param m_cde A numeric vector of length 1. Mediator level at which controlled direct effect is evaluated at.
-##' @param c_cond A numeric vector of the same length as \code{cvar}. Covariate vector at which conditional effects are evaluated at.
+##' @param c_cond A numeric vector of the same length as \code{cvar}. Covariate levels at which natural direct and indirect effects are evaluated at. 
 ##' @param mreg A character vector of length 1. Mediator regression type: \code{"linear"} or \code{"logistic"}.
 ##' @param yreg A character vector of length 1. Outcome regression type: \code{"linear"}, \code{"logistic"}, \code{"loglinear"}, \code{"poisson"}, \code{"negbin"}, \code{"survCox"}, \code{"survAFT_exp"}, or \code{"survAFT_weibull"}.
-##' @param interaction A logical vector of length 1. Default to TRUE. Whether to include a mediator-treatment interaction term in the outcome regression model.
+##' @param interaction A logical vector of length 1. The presence of treatment-mediator interaction in the outcome model. Default to TRUE.
 ##' @param casecontrol A logical vector of length 1. Default to FALSE. Whether data comes from a case-control study.
-##' @param na_omit A logical vector of length 1. Default to FALSE. Whether to use na.omit() function in stats package to remove NAs in columns of interest before fitting the models.
-##'
+##' @param na_omit A logical vector of length 1. Default to FALSE. Whether to remove NAs in the columns of interest before fitting the models.
+##' 
 ##' @return regmedint object, which is a list containing the mediator regression object, the outcome regression object, and the regression-based mediation results.
 ##'
 ##' @examples
 ##' library(regmedint)
 ##' data(vv2015)
-##' regmedint_obj <- regmedint(data = vv2015,
-##'                            ## Variables
-##'                            yvar = "y",
-##'                            avar = "x",
-##'                            mvar = "m",
-##'                            cvar = c("c"),
-##'                            eventvar = "event",
-##'                            ## Values at which effects are evaluated
-##'                            a0 = 0,
-##'                            a1 = 1,
-##'                            m_cde = 1,
-##'                            c_cond = 0.5,
-##'                            ## Model types
-##'                            mreg = "logistic",
-##'                            yreg = "survAFT_weibull",
-##'                            ## Additional specification
-##'                            interaction = TRUE,
-##'                            casecontrol = FALSE)
-##' summary(regmedint_obj)
+##' regmedint_obj1 <- regmedint(data = vv2015,
+##'                             ## Variables
+##'                             yvar = "y",
+##'                             avar = "x",
+##'                             mvar = "m",
+##'                             cvar = c("c"),
+##'                             eventvar = "event",
+##'                             ## Values at which effects are evaluated
+##'                             a0 = 0,
+##'                             a1 = 1,
+##'                             m_cde = 1,
+##'                             c_cond = 3,
+##'                             ## Model types
+##'                             mreg = "logistic",
+##'                             yreg = "survAFT_weibull",
+##'                             ## Additional specification
+##'                             interaction = TRUE,
+##'                             casecontrol = FALSE)
+##' summary(regmedint_obj1)
+##' 
+##' regmedint_obj2 <- regmedint(data = vv2015,
+##'                             ## Variables
+##'                             yvar = "y",
+##'                             avar = "x",
+##'                             mvar = "m",
+##'                             cvar = c("c"),
+##'                             emm_ac_mreg = c("c"), 
+##'                             emm_ac_yreg = c("c"), 
+##'                             emm_mc_yreg = c("c"), 
+##'                             eventvar = "event",
+##'                             ## Values at which effects are evaluated
+##'                             a0 = 0,
+##'                             a1 = 1,
+##'                             m_cde = 1,
+##'                             c_cond = 3,
+##'                             ## Model types
+##'                             mreg = "logistic",
+##'                             yreg = "survAFT_weibull",
+##'                             ## Additional specification
+##'                             interaction = TRUE,
+##'                             casecontrol = FALSE)
+##' summary(regmedint_obj2)
+##' 
+##' 
+##' 
 ##'
 ##' @export
 regmedint <- function(data,
@@ -71,6 +100,9 @@ regmedint <- function(data,
                       avar,
                       mvar,
                       cvar,
+                      emm_ac_mreg = NULL, 
+                      emm_ac_yreg = NULL, 
+                      emm_mc_yreg = NULL, 
                       eventvar = NULL,
                       a0,
                       a1,
@@ -81,10 +113,10 @@ regmedint <- function(data,
                       interaction = TRUE,
                       casecontrol = FALSE,
                       na_omit = FALSE) {
-
+    
     ## This is the user-friendly helper function with a name that is the class name.
     ## https://adv-r.hadley.nz/s3.html#helpers
-
+    
     ## Handle missing value
     ## Select columns of interest only.
     data <- data[,c(yvar, avar, mvar, cvar, eventvar)]
@@ -94,13 +126,16 @@ regmedint <- function(data,
     if(any(is.na(data)) && na_omit) {
         data <- na.omit(data)
     }
-
-    ## Check data contains yvar, avar, mvar, cvar, eventvar (if provided)
+    
+    ## Check data contains yvar, avar, mvar, cvar, eventvar (if provided), emm_ac_mreg, emm_ac_yreg, emm_mc_yreg
     validate_args(data = data,
                   yvar = yvar,
                   avar = avar,
                   mvar = mvar,
                   cvar = cvar,
+                  emm_ac_mreg = emm_ac_mreg, 
+                  emm_ac_yreg = emm_ac_yreg, 
+                  emm_mc_yreg = emm_mc_yreg, 
                   a0 = a0,
                   a1 = a1,
                   m_cde = m_cde,
@@ -110,7 +145,7 @@ regmedint <- function(data,
                   interaction = interaction,
                   casecontrol = casecontrol,
                   eventvar = eventvar)
-
+    
     ## Construct a regmedint object after argument validation.
     ## This is the low-level constructor function.
     ## https://adv-r.hadley.nz/s3.html#s3-constructor
@@ -119,6 +154,9 @@ regmedint <- function(data,
                          avar = avar,
                          mvar = mvar,
                          cvar = cvar,
+                         emm_ac_mreg = emm_ac_mreg, 
+                         emm_ac_yreg = emm_ac_yreg, 
+                         emm_mc_yreg = emm_mc_yreg, 
                          a0 = a0,
                          a1 = a1,
                          m_cde = m_cde,
@@ -128,12 +166,12 @@ regmedint <- function(data,
                          interaction = interaction,
                          casecontrol = casecontrol,
                          eventvar = eventvar)
-
+    
     ## Check the resulting object for anomalies.
     ## This is the class validator function.
     ## https://adv-r.hadley.nz/s3.html#validators
     validate_regmedint(res)
-
+    
     ## Return results
     res
 }
@@ -191,6 +229,9 @@ validate_args <- function(data,
                           avar,
                           mvar,
                           cvar,
+                          emm_ac_mreg, 
+                          emm_ac_yreg, 
+                          emm_mc_yreg, 
                           eventvar,
                           a0,
                           a1,
@@ -200,10 +241,10 @@ validate_args <- function(data,
                           yreg,
                           interaction,
                           casecontrol) {
-
+    
     ## Dataset
     assertthat::assert_that(is.data.frame(data))
-
+    
     ##
     assertthat::assert_that(is.character(yvar))
     assertthat::assert_that(length(yvar) == 1)
@@ -216,6 +257,12 @@ validate_args <- function(data,
     ##
     assertthat::assert_that(is.null(cvar) | is.character(cvar))
     ##
+    assertthat::assert_that(is.null(emm_ac_mreg) | is.character(emm_ac_mreg))
+    ##
+    assertthat::assert_that(is.null(emm_ac_yreg) | is.character(emm_ac_yreg))
+    ##
+    assertthat::assert_that(is.null(emm_mc_yreg) | is.character(emm_mc_yreg))
+    ##
     assertthat::assert_that(is.numeric(a0))
     assertthat::assert_that(length(a0) == 1)
     assertthat::assert_that(!is.na(a0))
@@ -250,7 +297,7 @@ validate_args <- function(data,
     ##
     assertthat::assert_that(is.null(eventvar) | is.character(eventvar))
     assertthat::assert_that(length(eventvar) <= 1)
-
+    
     ## Do not allow missing data in variables of interest
     ## because they may differ unexpected differences in sample sizes
     ## between yreg and mreg.
@@ -266,8 +313,8 @@ validate_args <- function(data,
 For multiple imputation, see the multiple imputation vignette: vignette(\"vig_04_mi\")
 To perform complete case analysis, use na_omit = TRUE.
 Variables with missingness: ",
-paste0(vars_missing, collapse = ", ")))
-
+                                         paste0(vars_missing, collapse = ", ")))
+    
     ## Do not allow factors as they can result in multiple
     ## dummy variables and coef results in different names
     ## from the original variable names.
@@ -276,7 +323,7 @@ paste0(vars_missing, collapse = ", ")))
                                         ## template value
                                         FUN.VALUE = TRUE)),
                             msg = "Factors are not allowed! Use numeric variables only. Create multiple dichotomous variables for multi-category variables.")
-
+    
     NULL
 }
 
@@ -293,13 +340,16 @@ paste0(vars_missing, collapse = ", ")))
 ##'
 ##' @return No return value, called for side effects.
 validate_regmedint <- function(x) {
-
+    
     assertthat::assert_that(class(x)[[1]] == "regmedint")
     ##
     assertthat::assert_that("myreg" %in% names(x))
     assertthat::assert_that("args" %in% names(x))
     ##
     assertthat::assert_that(is.list(x$args))
-
+    
     NULL
 }
+
+
+
