@@ -6,36 +6,43 @@
 ################################################################################
 
 
-## VanderWeele 2015 p473 Proposition 2.6
+## Extension of VanderWeele 2015 p473 Proposition 2.6
 ##' Create calculators for effects and se (mreg logistic / yreg logistic)
 ##'
-##' Construct functions for the conditional effect estimates and their standard errors in the mreg logistic / yreg logistic setting. Internally, this function deconstruct model objects and feed parameter estiamtes to the internal worker functions \code{calc_myreg_mreg_logistic_yreg_logistic_est} and \code{calc_myreg_mreg_logistic_yreg_logistic_se}.
+##' Construct functions for the conditional effect estimates and their standard errors in the mreg logistic / yreg logistic setting. Internally, this function deconstructs model objects and feeds parameter estimates to the internal worker functions \code{calc_myreg_mreg_logistic_yreg_logistic_est} and \code{calc_myreg_mreg_logistic_yreg_logistic_se}.
 ##'
 ##' @inheritParams regmedint
 ##' @param mreg_fit Model fit from \code{\link{fit_mreg}}
 ##' @param yreg_fit Model fit from \code{\link{fit_yreg}}
 ##'
-##' @return A list contraining a function for effect estimates and a function for corresponding standard errors.
+##' @return A list containing a function for effect estimates and a function for corresponding standard errors.
+
 calc_myreg_mreg_logistic_yreg_logistic <- function(mreg,
                                                    mreg_fit,
                                                    yreg,
                                                    yreg_fit,
                                                    avar,
                                                    mvar,
-                                                   cvar, # This can be NULL.
+                                                   cvar, 
+                                                   emm_ac_mreg,
+                                                   emm_ac_yreg,
+                                                   emm_mc_yreg,
                                                    interaction) {
 
     ## mreg coefficients
     beta_hat <- beta_hat_helper(mreg = mreg,
                                 mreg_fit = mreg_fit,
                                 avar = avar,
-                                cvar = cvar)
+                                cvar = cvar,
+                                emm_ac_mreg = emm_ac_mreg)
     ## yreg coefficients
     theta_hat <- theta_hat_helper(yreg = yreg,
                                   yreg_fit = yreg_fit,
                                   avar = avar,
                                   mvar = mvar,
                                   cvar = cvar,
+                                  emm_ac_yreg = emm_ac_yreg,
+                                  emm_mc_yreg = emm_mc_yreg,
                                   interaction = interaction)
     ## Construct a function of (a1, a0, m_cde, c_cond) that returns
     ## a vector of point estimates for quantities of interest.
@@ -43,22 +50,29 @@ calc_myreg_mreg_logistic_yreg_logistic <- function(mreg,
         calc_myreg_mreg_logistic_yreg_logistic_est(beta0 = beta_hat$beta0,
                                                    beta1 = beta_hat$beta1,
                                                    beta2 = beta_hat$beta2,
+                                                   beta3 = beta_hat$beta3,
                                                    theta0 = theta_hat$theta0,
                                                    theta1 = theta_hat$theta1,
                                                    theta2 = theta_hat$theta2,
                                                    theta3 = theta_hat$theta3,
-                                                   theta4 = theta_hat$theta4)
+                                                   theta4 = theta_hat$theta4,
+                                                   theta5 = theta_hat$theta5,
+                                                   theta6 = theta_hat$theta6
+                                                   )
 
     ## vcovs
     Sigma_beta_hat <- Sigma_beta_hat(mreg = mreg,
                                      mreg_fit = mreg_fit,
                                      avar = avar,
-                                     cvar = cvar)
+                                     cvar = cvar,
+                                     emm_ac_mreg = emm_ac_mreg)
     Sigma_theta_hat <- Sigma_theta_hat(yreg = yreg,
                                        yreg_fit = yreg_fit,
                                        avar = avar,
                                        mvar = mvar,
                                        cvar = cvar,
+                                       emm_ac_yreg = emm_ac_yreg,
+                                       emm_mc_yreg = emm_mc_yreg,
                                        interaction = interaction)
     ## Construct a function of (a0, a1, m_cde, c_cond) that returns
     ## a vector of estimates.
@@ -66,11 +80,14 @@ calc_myreg_mreg_logistic_yreg_logistic <- function(mreg,
         calc_myreg_mreg_logistic_yreg_logistic_se(beta0 = beta_hat$beta0,
                                                   beta1 = beta_hat$beta1,
                                                   beta2 = beta_hat$beta2,
+                                                  beta3 = beta_hat$beta3,
                                                   theta0 = theta_hat$theta0,
                                                   theta1 = theta_hat$theta1,
                                                   theta2 = theta_hat$theta2,
                                                   theta3 = theta_hat$theta3,
                                                   theta4 = theta_hat$theta4,
+                                                  theta5 = theta_hat$theta5,
+                                                  theta6 = theta_hat$theta6,
                                                   Sigma_beta = Sigma_beta_hat,
                                                   Sigma_theta = Sigma_theta_hat)
 
@@ -88,20 +105,26 @@ calc_myreg_mreg_logistic_yreg_logistic <- function(mreg,
 calc_myreg_mreg_logistic_yreg_logistic_est <- function(beta0,
                                                        beta1,
                                                        beta2,
+                                                       beta3,
                                                        theta0,
                                                        theta1,
                                                        theta2,
                                                        theta3,
-                                                       theta4) {
+                                                       theta4,
+                                                       theta5,
+                                                       theta6) {
 
     validate_myreg_coefs(beta0 = beta0,
                          beta1 = beta1,
                          beta2 = beta2,
+                         beta3 = beta3,
                          theta0 = theta0,
                          theta1 = theta1,
                          theta2 = theta2,
                          theta3 = theta3,
-                         theta4 = theta4)
+                         theta4 = theta4,
+                         theta5 = theta5,
+                         theta6 = theta6)
 
     ## Construct a function for point estimates given (a0, a1, m_cde, c_cond).
     fun_est <- function(a0, a1, m_cde, c_cond) {
@@ -116,32 +139,63 @@ calc_myreg_mreg_logistic_yreg_logistic_est <- function(beta0,
             assertthat::assert_that(length(c_cond) == length(beta2))
             beta2_c <- sum(t(matrix(beta2)) %*% matrix(c_cond))
         }
+        
+        if (is.null(beta3)) {
+            beta3_c <- 0
+        } else {
+            assertthat::assert_that(length(c_cond) == length(beta3))
+            beta3_c <- sum(t(matrix(beta3)) %*% matrix(c_cond))
+        }
+        
+        if (is.null(theta4)) {
+            assertthat::assert_that(is.null(c_cond))
+            theta4_c <- 0
+        } else {
+            assertthat::assert_that(!is.null(c_cond))
+            assertthat::assert_that(length(c_cond) == length(theta4))
+            theta4_c <- sum(t(matrix(theta4)) %*% matrix(c_cond))
+        }
+        
+        if (is.null(theta5)) {
+            theta5_c <- 0
+        } else {
+            assertthat::assert_that(length(c_cond) == length(theta5))
+            theta5_c <- sum(t(matrix(theta5)) %*% matrix(c_cond))
+        }
+        
+        if (is.null(theta6)) {
+            theta6_c <- 0
+        } else {
+            assertthat::assert_that(length(c_cond) == length(theta6))
+            theta6_c <- sum(t(matrix(theta6)) %*% matrix(c_cond))
+        }
+        
+        expit <- function(x){exp(x)/(1+exp(x))}
 
-        ## VanderWeele 2015 p473
-        ## Note that only cde is presented on the log scale.
-        cde <- (theta1 + (theta3 * m_cde)) * (a1 - a0)
+        ## Extension of VanderWeele 2015 p473. Estimates are all on log OR scale.
+        ## Note that only cde is presented on the log scale
+        cde <- (theta1 + theta3*m_cde + theta5_c) * (a1 - a0)
         ## Pearl decomposition (Regular NDE and NIE)
         ## Note the a0 in the second term.
-        pnde <- (theta1 * (a1 - a0)) +
-            log(1 + exp(theta2 + (theta3 * a1) + beta0 + (beta1 * a0) + beta2_c)) -
-            log(1 + exp(theta2 + (theta3 * a0) + beta0 + (beta1 * a0) + beta2_c))
+        pnde <- ((theta1 + theta5_c) * (a1 - a0)) +
+            log(1 + exp(beta0 + beta1*a0 + beta2_c + beta3_c*a0 + theta2 + theta3*a1 + theta6_c)) -
+            log(1 + exp(beta0 + beta1*a0 + beta2_c + beta3_c*a0 + theta2 + theta3*a0 + theta6_c))
         ## Note the a1 in the first term.
-        tnie <-
-            log(1 + exp(beta0 + (beta1 * a0) + beta2_c)) -
-            log(1 + exp(beta0 + (beta1 * a1) + beta2_c)) +
-            log(1 + exp(theta2 + (theta3 * a1) + beta0 + (beta1 * a1) + beta2_c)) -
-            log(1 + exp(theta2 + (theta3 * a1) + beta0 + (beta1 * a0) + beta2_c))
+        tnie <- log(1 + exp(beta0 + beta1*a1 + beta2_c + beta3_c*a1 + theta2 + theta3*a1 + theta6_c)) - 
+            log(1 + exp(beta0 + beta1*a0 + beta2_c + beta3_c*a0 + theta2 + theta3*a1 + theta6_c)) +
+            log(1 + exp(beta0 + beta1*a0 + beta2_c + beta3_c*a0)) - 
+            log(1 + exp(beta0 + beta1*a1 + beta2_c + beta3_c*a1))
         ## Another decomposition
         ## Note the a0 -> a1 changes associated with beta1.
-        tnde <- (theta1 * (a1 - a0)) +
-            log(1 + exp(theta2 + (theta3 * a1) + beta0 + (beta1 * a1) + beta2_c)) -
-            log(1 + exp(theta2 + (theta3 * a0) + beta0 + (beta1 * a1) + beta2_c))
+        tnde <- ((theta1 + theta5_c) * (a1 - a0)) +
+            log(1 + exp(beta0 + beta1*a1 + beta2_c + beta3_c*a1 + theta2 + theta3*a1 + theta6_c)) -
+            log(1 + exp(beta0 + beta1*a1 + beta2_c + beta3_c*a1 + theta2 + theta3*a0 + theta6_c))
         ## Note the a1 -> a0 changes associated with theta3.
         pnie <-
-            log(1 + exp(beta0 + (beta1 * a0) + beta2_c)) -
-            log(1 + exp(beta0 + (beta1 * a1) + beta2_c)) +
-            log(1 + exp(theta2 + (theta3 * a0) + beta0 + (beta1 * a1) + beta2_c)) -
-            log(1 + exp(theta2 + (theta3 * a0) + beta0 + (beta1 * a0) + beta2_c))
+            log(1 + exp(beta0 + beta1*a1 + beta2_c + beta3_c*a1 + theta2 + theta3*a0 + theta6_c)) - 
+            log(1 + exp(beta0 + beta1*a0 + beta2_c + beta3_c*a0 + theta2 + theta3*a0 + theta6_c)) +
+            log(1 + exp(beta0 + beta1*a0 + beta2_c + beta3_c*a0)) - 
+            log(1 + exp(beta0 + beta1*a1 + beta2_c + beta3_c*a1))
         ## It is the sum of NDE and NIE on the log scale.
         te <- pnde + tnie
         ## VanderWeele 2015 p48.
@@ -165,31 +219,40 @@ calc_myreg_mreg_logistic_yreg_logistic_est <- function(beta0,
 calc_myreg_mreg_logistic_yreg_logistic_se <- function(beta0,
                                                       beta1,
                                                       beta2,
+                                                      beta3,
                                                       theta0,
                                                       theta1,
                                                       theta2,
                                                       theta3,
                                                       theta4,
+                                                      theta5,
+                                                      theta6,
                                                       Sigma_beta,
                                                       Sigma_theta) {
 
     validate_myreg_coefs(beta0 = beta0,
                          beta1 = beta1,
                          beta2 = beta2,
-                         theta0 = theta0,
-                         theta1 = theta1,
-                         theta2 = theta2,
-                         theta3 = theta3,
-                         theta4 = theta4)
-
-    validate_myreg_vcovs(beta0 = beta0,
-                         beta1 = beta1,
-                         beta2 = beta2,
+                         beta3 = beta3,
                          theta0 = theta0,
                          theta1 = theta1,
                          theta2 = theta2,
                          theta3 = theta3,
                          theta4 = theta4,
+                         theta5 = theta5,
+                         theta6 = theta6)
+
+    validate_myreg_vcovs(beta0 = beta0,
+                         beta1 = beta1,
+                         beta2 = beta2,
+                         beta3 = beta3,
+                         theta0 = theta0,
+                         theta1 = theta1,
+                         theta2 = theta2,
+                         theta3 = theta3,
+                         theta4 = theta4,
+                         theta5 = theta5,
+                         theta6 = theta6,
                          Sigma_beta = Sigma_beta,
                          Sigma_theta = Sigma_theta)
 
@@ -209,128 +272,243 @@ calc_myreg_mreg_logistic_yreg_logistic_se <- function(beta0,
             assertthat::assert_that(length(c_cond) == length(beta2))
             beta2_c <- sum(t(matrix(beta2)) %*% matrix(c_cond))
         }
+        
+        if (is.null(beta3)) {
+            beta3_c <- 0
+        } else {
+            assertthat::assert_that(length(c_cond) == length(beta3))
+            beta3_c <- sum(t(matrix(beta3)) %*% matrix(c_cond))
+        }
+        
+        if (is.null(theta4)) {
+            assertthat::assert_that(is.null(c_cond))
+            theta4_c <- 0
+        } else {
+            assertthat::assert_that(!is.null(c_cond))
+            assertthat::assert_that(length(c_cond) == length(theta4))
+            theta4_c <- sum(t(matrix(theta4)) %*% matrix(c_cond))
+        }
+        
+        if (is.null(theta5)) {
+            theta5_c <- 0
+        } else {
+            assertthat::assert_that(length(c_cond) == length(theta5))
+            theta5_c <- sum(t(matrix(theta5)) %*% matrix(c_cond))
+        }
+        
+        if (is.null(theta6)) {
+            theta6_c <- 0
+        } else {
+            assertthat::assert_that(length(c_cond) == length(theta6))
+            theta6_c <- sum(t(matrix(theta6)) %*% matrix(c_cond))
+        }
+        
+        expit <- function(x){exp(x)/(1+exp(x))}
 
-        ## VanderWeele 2015. p473
-        ## Note that Gammas are all for log effects.
+        ## Extension of VanderWeele 2015. p473
+        ## Note that Gammas are all for log OR effects.
         ## Valeri & VanderWeele 2013. Appendix p6-9
         ## These are the gradient vector of each scalar quantity of interest.
         ## Obtain the first partial derivative wrt to each parameter.
+        if(is.null(theta5)){
+            pd_cde_theta5 <- rep(0, length(theta5))
+        }else{
+            pd_cde_theta5 <- c_cond
+        }
         Gamma_cde <-
             matrix(c(0,                        # beta0
                      0,                        # beta1
                      rep(0, length(beta2)),    # beta2 vector
+                     rep(0, length(beta3)),    # beta3 vector
                      ##
                      0,                        # theta0
                      (a1 - a0),                # theta1
                      0,                        # theta2
                      (a1 - a0) * m_cde,        # theta3
-                     rep(0, length(theta4))))  # theta4 vector
+                     rep(0, length(theta4)),   # theta4 vector
+                     pd_cde_theta5,            # theta5 vector
+                     rep(0, length(theta6))    # theta6 vector
+                     ))  
         ##
-        pnde_Q <- exp(theta2 + (theta3 * a1) + beta0 + (beta1 * a0) + beta2_c) /
-            (1  + exp(theta2 + (theta3 * a1) + beta0 + (beta1 * a0) + beta2_c))
-        pnde_B <- exp(theta2 + (theta3 * a0) + beta0 + (beta1 * a0) + beta2_c) /
-            (1  + exp(theta2 + (theta3 * a0) + beta0 + (beta1 * a0) + beta2_c))
-        pnde_d1 <- pnde_Q - pnde_B
-        pnde_d2 <- a0 * pnde_d1
-        pnde_d3 <- c_cond * pnde_d1
-        pnde_d4 <- 0
-        pnde_d5 <- (a1 - a0)
-        pnde_d6 <- pnde_Q - pnde_B
-        pnde_d7 <- (a1 * pnde_Q) - (a0 * pnde_B)
-        pnde_d8 <- rep(0, length(theta4))
+        pnde_expit_a1 <- expit(a0*(beta1 + beta3_c) + a1*theta3 + beta0 + beta2_c + theta6_c + theta2)
+        pnde_expit_a0 <- expit(a0*(beta1 + beta3_c) + a0*theta3 + beta0 + beta2_c + theta6_c + theta2)
+        
+        pnde_d1 <- pnde_expit_a1 - pnde_expit_a0
+        pnde_d2 <- a0*pnde_d1
+        pnde_d3 <- c_cond*pnde_d1
+        # pnde_d4 <- c_cond*a0*pnde_d1
+        if(is.null(beta3)){
+            pnde_d4 <- rep(0, length(beta3))
+        }else{
+            pnde_d4 <- c_cond*a0*pnde_d1
+        }
+        pnde_d5 <- 0
+        pnde_d6 <- a1 - a0
+        pnde_d7 <- pnde_d1
+        pnde_d8 <- a1*pnde_expit_a1 - a0*pnde_expit_a0
+        pnde_d9 <- rep(0, length(theta4))
+        # pnde_d10 <- c_cond * (a1 - a0)
+        if(is.null(theta5)){
+            pnde_d10 <- rep(0, length(theta5))
+        }else{
+            pnde_d10 <- c_cond*(a1 - a0)
+        }
+        # pnde_d11 <- c_cond*pnde_d1
+        if(is.null(theta6)){
+            pnde_d11 <- rep(0, length(theta6))
+        }else{
+            pnde_d11 <- c_cond*pnde_d1
+        }
+        
         Gamma_pnde <-
             matrix(c(
                 pnde_d1,   # beta0
                 pnde_d2,   # beta1
                 pnde_d3,   # beta2 vector
+                pnde_d4,   # beta3 vector
                 ##
-                pnde_d4,   # theta0
-                pnde_d5,   # theta1
-                pnde_d6,   # theta2
-                pnde_d7,   # theta3
-                pnde_d8))  # theta4 vector
+                pnde_d5,   # theta0
+                pnde_d6,   # theta1
+                pnde_d7,   # theta2
+                pnde_d8,   # theta3
+                pnde_d9,   # theta4 vector
+                pnde_d10,  # theta5 vector
+                pnde_d11   # theta6 vector
+                ))  
         ##
-        tnie_Q <- exp(theta2 + (theta3 * a1) + beta0 + (beta1 * a1) + beta2_c) /
-            (1  + exp(theta2 + (theta3 * a1) + beta0 + (beta1 * a1) + beta2_c))
-        tnie_B <- exp(theta2 + (theta3 * a1) + beta0 + (beta1 * a0) + beta2_c) /
-            (1  + exp(theta2 + (theta3 * a1) + beta0 + (beta1 * a0) + beta2_c))
-        tnie_K <- exp(beta0 + (beta1 * a1) + beta2_c) /
-            (1 +  exp(beta0 + (beta1 * a1) + beta2_c))
-        tnie_D <- exp(beta0 + (beta1 * a0) + beta2_c) /
-            (1 +  exp(beta0 + (beta1 * a0) + beta2_c))
-        tnie_d1 <- (tnie_D + tnie_Q) - (tnie_K + tnie_B)
-        tnie_d2 <- (a0 * (tnie_D - tnie_B)) + (a1 * (tnie_Q - tnie_K))
-        tnie_d3 <- c_cond * ((tnie_D + tnie_Q) - (tnie_K + tnie_B))
-        tnie_d4 <- 0
+        tnie_expit_q1 <- expit(a0*(beta1 + beta3_c) + beta0 + beta2_c)
+        tnie_expit_q2 <- expit(a0*(beta1 + beta3_c) + a1*theta3 + beta0 + beta2_c + theta6_c + theta2)
+        tnie_expit_q3 <- expit(a1*(beta1 + beta3_c) + beta0 + beta2_c)
+        tnie_expit_q4 <- expit(a1*(beta1 + beta3_c) + a1*theta3 + beta0 + beta2_c + theta6_c + theta2)
+        
+        tnie_d1 <- tnie_expit_q1 - tnie_expit_q2 - tnie_expit_q3 + tnie_expit_q4
+        tnie_d2 <- a0 * (tnie_expit_q1-tnie_expit_q2) + a1 * (-tnie_expit_q3+tnie_expit_q4)
+        tnie_d3 <- c_cond*tnie_d1
+        # tnie_d4 <- c_cond*tnie_d2
+        if(is.null(beta3)){
+            tnie_d4 <- rep(0, length(beta3))
+        }else{
+            tnie_d4 <- c_cond*tnie_d2
+        }
         tnie_d5 <- 0
-        tnie_d6 <- tnie_Q - tnie_B
-        tnie_d7 <- a1 * (tnie_Q - tnie_B)
-        tnie_d8 <- rep(0, length(theta4))
+        tnie_d6 <- 0
+        tnie_d7 <- tnie_expit_q4 - tnie_expit_q2  # watch out the order
+        tnie_d8 <- a1*tnie_d7
+        tnie_d9 <- rep(0, length(theta4))
+        tnie_d10 <- rep(0, length(theta5))
+        # tnie_d11 <- c_cond*tnie_d7 
+        if(is.null(theta6)){
+            tnie_d11 <- rep(0, length(theta6))
+        }else{
+            tnie_d11 <- c_cond*tnie_d7
+        }
+        
         Gamma_tnie <-
             matrix(c(
                 tnie_d1,   # beta0
                 tnie_d2,   # beta1
                 tnie_d3,   # beta2 vector
+                tnie_d4,   # beta3 vector
                 ##
-                tnie_d4,   # theta0
-                tnie_d5,   # theta1
-                tnie_d6,   # theta2
-                tnie_d7,   # theta3
-                tnie_d8))  # theta4 vector
+                tnie_d5,   # theta0
+                tnie_d6,   # theta1
+                tnie_d7,   # theta2
+                tnie_d8,   # theta3
+                tnie_d9,   # theta4 vector
+                tnie_d10,  # theta5 vector
+                tnie_d11   # theta6 vector
+            ))
         ##
         ## a's from mreg beta1 should be a1.
-        tnde_Q <- exp(theta2 + (theta3 * a1) + beta0 + (beta1 * a1) + beta2_c) /
-            (1  + exp(theta2 + (theta3 * a1) + beta0 + (beta1 * a1) + beta2_c))
-        tnde_B <- exp(theta2 + (theta3 * a0) + beta0 + (beta1 * a1) + beta2_c) /
-            (1  + exp(theta2 + (theta3 * a0) + beta0 + (beta1 * a1) + beta2_c))
-        tnde_d1 <- tnde_Q - tnde_B
-        tnde_d2 <- a1 * tnde_d1
-        tnde_d3 <- c_cond * tnde_d1
-        tnde_d4 <- 0
-        tnde_d5 <- (a1 - a0)
-        tnde_d6 <- tnde_Q - tnde_B
-        tnde_d7 <- (a1 * tnde_Q) - (a0 * tnde_B)
-        tnde_d8 <- rep(0, length(theta4))
+        tnde_expit_a1 <- expit(a1*(beta1+beta3_c) + a1*theta3 + beta0 + beta2_c + theta6_c + theta2)
+        tnde_expit_a0 <- expit(a1*(beta1+beta3_c) + a0*theta3 + beta0 + beta2_c + theta6_c + theta2)
+        
+        tnde_d1 <- tnde_expit_a1 - tnde_expit_a0
+        tnde_d2 <- a1*tnde_d1
+        tnde_d3 <- c_cond*tnde_d1
+        # tnde_d4 <- c_cond*a1*tnde_d1
+        if(is.null(beta3)){
+            tnde_d4 <- rep(0, length(beta3))
+        }else{
+            tnde_d4 <- c_cond*a1*tnde_d1
+        }
+        tnde_d5 <- 0
+        tnde_d6 <- a1 - a0
+        tnde_d7 <- tnde_d1
+        tnde_d8 <- a1*tnde_expit_a1 - a0*tnde_expit_a0
+        tnde_d9 <- rep(0, length(theta4))
+        # tnde_d10 <- c_cond*(a1 - a0)
+        if(is.null(theta5)){
+            tnde_d10 <- rep(0, length(theta5))
+        }else{
+            tnde_d10 <- c_cond*(a1 - a0)
+        }
+        # tnde_d11 <- tnde_d3
+        if(is.null(theta6)){
+            tnde_d11 <- rep(0, length(theta6))
+        }else{
+            tnde_d11 <- tnde_d3
+        }
+        
         Gamma_tnde <-
             matrix(c(
                 tnde_d1,   # beta0
                 tnde_d2,   # beta1
                 tnde_d3,   # beta2 vector
+                tnde_d4,   # beta3 vector
                 ##
-                tnde_d4,   # theta0
-                tnde_d5,   # theta1
-                tnde_d6,   # theta2
-                tnde_d7,   # theta3
-                tnde_d8))  # theta4 vector
+                tnde_d5,   # theta0
+                tnde_d6,   # theta1
+                tnde_d7,   # theta2
+                tnde_d8,   # theta3
+                tnde_d9,   # theta4 vector
+                tnde_d10,  # theta5 vector
+                tnde_d11   # theta6 vector
+                ))  
         ##
         ## a's from yreg theta3 should be a0.
-        pnie_Q <- exp(theta2 + (theta3 * a0) + beta0 + (beta1 * a1) + beta2_c) /
-            (1  + exp(theta2 + (theta3 * a0) + beta0 + (beta1 * a1) + beta2_c))
-        pnie_B <- exp(theta2 + (theta3 * a0) + beta0 + (beta1 * a0) + beta2_c) /
-            (1  + exp(theta2 + (theta3 * a0) + beta0 + (beta1 * a0) + beta2_c))
-        pnie_K <- exp(beta0 + (beta1 * a1) + beta2_c) /
-            (1 +  exp(beta0 + (beta1 * a1) + beta2_c))
-        pnie_D <- exp(beta0 + (beta1 * a0) + beta2_c) /
-            (1 +  exp(beta0 + (beta1 * a0) + beta2_c))
-        pnie_d1 <- (pnie_D + pnie_Q) - (pnie_K + pnie_B)
-        pnie_d2 <- (a0 * (pnie_D - pnie_B)) + (a1 * (pnie_Q - pnie_K))
-        pnie_d3 <- c_cond * ((pnie_D + pnie_Q) - (pnie_K + pnie_B))
-        pnie_d4 <- 0
+        pnie_expit_q1 <- expit(a0*(beta1+beta3_c) + beta0 + beta2_c)
+        pnie_expit_q2 <- expit(a0*(beta1+beta3_c) + a0*theta3 + beta0 + beta2_c + theta6_c + theta2)
+        pnie_expit_q3 <- expit(a1*(beta1+beta3_c) + beta0 + beta2_c)
+        pnie_expit_q4 <- expit(a1*(beta1+beta3_c) + a0*theta3 + beta0 + beta2_c + theta6_c + theta2)
+        
+        pnie_d1 <- pnie_expit_q1 - pnie_expit_q2 - pnie_expit_q3 + pnie_expit_q4
+        pnie_d2 <- a0 * (pnie_expit_q1 - pnie_expit_q2) + a1 * (- pnie_expit_q3 + pnie_expit_q4)
+        pnie_d3 <- c_cond * pnie_d1
+
+        if(is.null(beta3)){
+            pnie_d4 <- rep(0, length(beta3))
+        }else{
+            pnie_d4 <- c_cond * pnie_d2
+        }
         pnie_d5 <- 0
-        pnie_d6 <- pnie_Q - pnie_B
-        pnie_d7 <- a0 * (pnie_Q - pnie_B)
-        pnie_d8 <- rep(0, length(theta4))
+        pnie_d6 <- 0
+        pnie_d7 <- pnie_expit_q4 - pnie_expit_q2
+        pnie_d8 <- a0 * pnie_d7 # change from a1
+        pnie_d9 <- rep(0, length(theta4))
+        pnie_d10 <- rep(0, length(theta5))
+        
+        if(is.null(theta6)){
+            pnie_d11 <- rep(0, length(theta6))
+        }else{
+            pnie_d11 <- c_cond * pnie_d7
+        }
+        
         Gamma_pnie <-
             matrix(c(
                 pnie_d1,   # beta0
                 pnie_d2,   # beta1
                 pnie_d3,   # beta2 vector
+                pnie_d4,   # beta3 vector
                 ##
-                pnie_d4,   # theta0
-                pnie_d5,   # theta1
-                pnie_d6,   # theta2
-                pnie_d7,   # theta3
-                pnie_d8))  # theta4 vector
+                pnie_d5,   # theta0
+                pnie_d6,   # theta1
+                pnie_d7,   # theta2
+                pnie_d8,   # theta3
+                pnie_d9,   # theta4 vector
+                pnie_d10,  # theta5 vector
+                pnie_d11   # theta6 vector
+                ))  
         ## because Gamma_pnie does not have a common factor.
         Gamma_te <-
             Gamma_pnde + Gamma_tnie # By linearity of differentiation
@@ -340,11 +518,14 @@ calc_myreg_mreg_logistic_yreg_logistic_se <- function(beta0,
         ## Gradient of pm wrt pnde and tnie. A vector of two.
         ## Copied from calc_myreg_mreg_logistic_yreg_logistic_est
         pnde <-
-            (exp(theta1 * a1) * (1 + exp(theta2 + (theta3 * a1) + beta0 + (beta1 * a0) + beta2_c))) /
-            (exp(theta1 * a0) * (1 + exp(theta2 + (theta3 * a0) + beta0 + (beta1 * a0) + beta2_c)))
+            ((theta1 + theta5_c) * (a1 - a0)) +
+            log(1 + exp(beta0 + beta1*a0 + beta2_c + beta3_c*a0 + theta2 + theta3*a1 + theta6_c)) -
+            log(1 + exp(beta0 + beta1*a0 + beta2_c + beta3_c*a0 + theta2 + theta3*a0 + theta6_c))
         tnie <-
-            ((1 + exp(beta0 + (beta1 * a0) + beta2_c)) * (1 + exp(theta2 + (theta3 * a1) + beta0 + (beta1 * a1) + beta2_c))) /
-            ((1 + exp(beta0 + (beta1 * a1) + beta2_c)) * (1 + exp(theta2 + (theta3 * a1) + beta0 + (beta1 * a0) + beta2_c)))
+            log(1 + exp(beta0 + beta1*a1 + beta2_c + beta3_c*a1 + theta2 + theta3*a1 + theta6_c)) - 
+            log(1 + exp(beta0 + beta1*a0 + beta2_c + beta3_c*a0 + theta2 + theta3*a1 + theta6_c)) +
+            log(1 + exp(beta0 + beta1*a0 + beta2_c + beta3_c*a0)) - 
+            log(1 + exp(beta0 + beta1*a1 + beta2_c + beta3_c*a1))
         ## Need to unname argument vectors to get c(pnde = , tnie = ).
         d_pm <- grad_prop_med_yreg_logistic(pnde = unname(pnde), tnie = unname(tnie))
         ## Multivariate chain rule.
